;===================================================
; This is the MASM 6+ source code for a QuickBASIC
; add on that maintains a count of System Timer
; Ticks in a QB variable.
;
; The QB variable must be of type LONG, and the
; segment and offset addresses of the variable
; must be passed in the first call to the
; TickCountInit procedure.
;===================================================

; This prototype establishes the call interface for
; the B_OnExit procedure and effectively generates
; an external declaration in this module.
B_OnExit proto far basic :dword

; This prototype establishes the call interface for
; TermProc.
TermProc proto far basic

.model medium,basic
.386

; Start a near data segment. The linker will combine
; this segment with the QB default data segment (but
; the variable will not be accessible from the QB
; program).
.data

   ; Allocate a flag variable that will be used to
   ; lock out multiple calls to TickCountInit.   
   f_init      dw 0

; Start a (far) code segment.
.code

   ; Allocate variables to store the far address of
   ; the QB tick count variable and the far address
   ; of the previous User Tick handler.
   tickCountPtr   dd 0
   oldUserTick    dd 0

;===================================================
; This proc saves the far address of the QB tick
; count variable, installs the new User Tick
; handler, and calls the B_OnExit routine to log a
; termination procedure that will automatically
; restore the previous User Tick handler when the
; program terminates. The first call sets a flag
; that locks out subsequent calls.
;
; This procedure ignores any error return from the
; B_OnExit routine on the assumption that fewer than
; 32 termination procedures will have been logged.
;===================================================
TickCountInit proc varSeg:WORD,varPtr:WORD

      ; Lock out subseqent calls.
      .IF f_init != 0
         xor   ax,ax
         ret
      .ENDIF
      mov   f_init,-1

      ; Save the address of QB tick count variable.
      mov   bx,varPtr
      mov   ax,[bx]
      mov   word ptr cs:tickCountPtr,ax
      mov   bx,varSeg
      mov   ax,[bx]
      mov   word ptr cs:tickCountPtr+2,ax
      
      ; Save the previous User Tick vector.
      mov   ax,351Ch
      int   21h
      mov   word ptr oldUserTick,bx
      mov   word ptr oldUserTick+2,es

      ; Set the vector to the address of
      ; the new handler.
      push  ds
      push  cs
      pop   ds
      mov   dx,offset UserTickHandler      
      mov   ax,251Ch
      int   21h
      pop   ds     

      ; Register the termination procedure.
      invoke B_OnExit,TermProc     
      
      ret
TickCountInit endp

;===================================================
; This is B_OnExit termination procedure. It simply
; restores the previous User Tick handler.
;===================================================
TermProc proc
      push  ds
      lds   dx,oldUserTick
      mov   ax,251Ch
      int   21h
      pop   ds
      ret
TermProc endp

;===================================================
; This proc is the User Tick handler. It simply
; increments the QB tick count variable and chains
; to the previous handler.
;
; A cs: override is not necessary because MASM knows
; that tickCountPtr and oldUserTick are in the code
; segment, so the override is encoded automatically.
;===================================================
UserTickHandler proc
      push  bx
      push  ds
      pushf
      lds   bx,tickCountPtr
      inc   dword ptr [bx]
      popf
      pop   ds
      pop   bx
      jmp   oldUserTick
UserTickHandler endp

end
